# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/settings.ui'
#
# Created by: PyQt5 UI code generator 5.7.1
#
# WARNING! All changes made in this file will be lost!

from ..qt import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(425, 626)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setWordWrap(False)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.build = QtWidgets.QComboBox(Dialog)
        self.build.setObjectName("build")
        self.horizontalLayout_3.addWidget(self.build)
        self.browseButton = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.browseButton.sizePolicy().hasHeightForWidth())
        self.browseButton.setSizePolicy(sizePolicy)
        self.browseButton.setObjectName("browseButton")
        self.horizontalLayout_3.addWidget(self.browseButton)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.line = QtWidgets.QFrame(Dialog)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.formLayout_4 = QtWidgets.QFormLayout()
        self.formLayout_4.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_4.setObjectName("formLayout_4")
        self.resolutionLabel = QtWidgets.QLabel(Dialog)
        self.resolutionLabel.setObjectName("resolutionLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.resolutionLabel)
        self.vid_res = QtWidgets.QComboBox(Dialog)
        self.vid_res.setObjectName("vid_res")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.vid_res)
        self.depthLabel = QtWidgets.QLabel(Dialog)
        self.depthLabel.setObjectName("depthLabel")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.depthLabel)
        self.vid_depth = QtWidgets.QComboBox(Dialog)
        self.vid_depth.setObjectName("vid_depth")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.vid_depth)
        self.horizontalLayout_2.addLayout(self.formLayout_4)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.textureFilterLabel = QtWidgets.QLabel(Dialog)
        self.textureFilterLabel.setObjectName("textureFilterLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.textureFilterLabel)
        self.vid_texfilter = QtWidgets.QComboBox(Dialog)
        self.vid_texfilter.setObjectName("vid_texfilter")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.vid_texfilter)
        self.antialiasingLabel = QtWidgets.QLabel(Dialog)
        self.antialiasingLabel.setObjectName("antialiasingLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.antialiasingLabel)
        self.vid_aa = QtWidgets.QComboBox(Dialog)
        self.vid_aa.setObjectName("vid_aa")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.vid_aa)
        self.anisotropicFilterLabel = QtWidgets.QLabel(Dialog)
        self.anisotropicFilterLabel.setObjectName("anisotropicFilterLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.anisotropicFilterLabel)
        self.vid_af = QtWidgets.QComboBox(Dialog)
        self.vid_af.setObjectName("vid_af")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.vid_af)
        self.horizontalLayout_2.addLayout(self.formLayout)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.line_3 = QtWidgets.QFrame(Dialog)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.label_3 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.formLayout_3 = QtWidgets.QFormLayout()
        self.formLayout_3.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_3.setObjectName("formLayout_3")
        self.playbackDeviceLabel = QtWidgets.QLabel(Dialog)
        self.playbackDeviceLabel.setObjectName("playbackDeviceLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.playbackDeviceLabel)
        self.snd_playback = QtWidgets.QComboBox(Dialog)
        self.snd_playback.setObjectName("snd_playback")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.snd_playback)
        self.captureDeviceLabel = QtWidgets.QLabel(Dialog)
        self.captureDeviceLabel.setObjectName("captureDeviceLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.captureDeviceLabel)
        self.snd_capture = QtWidgets.QComboBox(Dialog)
        self.snd_capture.setObjectName("snd_capture")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.snd_capture)
        self.verticalLayout.addLayout(self.formLayout_3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.snd_efx = QtWidgets.QCheckBox(Dialog)
        self.snd_efx.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.snd_efx.setObjectName("snd_efx")
        self.horizontalLayout_5.addWidget(self.snd_efx)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.sampleRateLabel = QtWidgets.QLabel(Dialog)
        self.sampleRateLabel.setObjectName("sampleRateLabel")
        self.horizontalLayout_5.addWidget(self.sampleRateLabel)
        self.snd_samplerate = QtWidgets.QSpinBox(Dialog)
        self.snd_samplerate.setObjectName("snd_samplerate")
        self.horizontalLayout_5.addWidget(self.snd_samplerate)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.line_4 = QtWidgets.QFrame(Dialog)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout.addWidget(self.line_4)
        self.keyboardLabel = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.keyboardLabel.setFont(font)
        self.keyboardLabel.setObjectName("keyboardLabel")
        self.verticalLayout.addWidget(self.keyboardLabel)
        self.keyboardForm = QtWidgets.QFormLayout()
        self.keyboardForm.setObjectName("keyboardForm")
        self.label_8 = QtWidgets.QLabel(Dialog)
        self.label_8.setObjectName("label_8")
        self.keyboardForm.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.setxkbmapCheck = QtWidgets.QCheckBox(Dialog)
        self.setxkbmapCheck.setObjectName("setxkbmapCheck")
        self.keyboardForm.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.setxkbmapCheck)
        self.keyLayoutSelect = QtWidgets.QComboBox(Dialog)
        self.keyLayoutSelect.setObjectName("keyLayoutSelect")
        self.keyboardForm.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.keyLayoutSelect)
        self.verticalLayout.addLayout(self.keyboardForm)
        self.line_7 = QtWidgets.QFrame(Dialog)
        self.line_7.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.verticalLayout.addWidget(self.line_7)
        self.label_4 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.ctrl_joystick = QtWidgets.QComboBox(Dialog)
        self.ctrl_joystick.setObjectName("ctrl_joystick")
        self.verticalLayout.addWidget(self.ctrl_joystick)
        self.line_6 = QtWidgets.QFrame(Dialog)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.verticalLayout.addWidget(self.line_6)
        self.label_6 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.verticalLayout.addWidget(self.label_6)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_2.setObjectName("formLayout_2")
        self.connectionTypeLabel = QtWidgets.QLabel(Dialog)
        self.connectionTypeLabel.setObjectName("connectionTypeLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.connectionTypeLabel)
        self.net_type = QtWidgets.QComboBox(Dialog)
        self.net_type.setObjectName("net_type")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.net_type)
        self.connectionSpeedLabel = QtWidgets.QLabel(Dialog)
        self.connectionSpeedLabel.setObjectName("connectionSpeedLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.connectionSpeedLabel)
        self.net_speed = QtWidgets.QComboBox(Dialog)
        self.net_speed.setObjectName("net_speed")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.net_speed)
        self.horizontalLayout_4.addLayout(self.formLayout_2)
        self.formLayout_6 = QtWidgets.QFormLayout()
        self.formLayout_6.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_6.setObjectName("formLayout_6")
        self.forceLocalPortLabel = QtWidgets.QLabel(Dialog)
        self.forceLocalPortLabel.setObjectName("forceLocalPortLabel")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.forceLocalPortLabel)
        self.net_port = QtWidgets.QLineEdit(Dialog)
        self.net_port.setObjectName("net_port")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.net_port)
        self.forceIPAddressLabel = QtWidgets.QLabel(Dialog)
        self.forceIPAddressLabel.setObjectName("forceIPAddressLabel")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.forceIPAddressLabel)
        self.net_ip = QtWidgets.QLineEdit(Dialog)
        self.net_ip.setObjectName("net_ip")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.net_ip)
        self.horizontalLayout_4.addLayout(self.formLayout_6)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.line_5 = QtWidgets.QFrame(Dialog)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.verticalLayout.addWidget(self.line_5)
        self.label_5 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.verticalLayout.addWidget(self.label_5)
        self.cmdButton = QtWidgets.QPushButton(Dialog)
        self.cmdButton.setObjectName("cmdButton")
        self.verticalLayout.addWidget(self.cmdButton)
        self.line_2 = QtWidgets.QFrame(Dialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.runButton = QtWidgets.QPushButton(Dialog)
        self.runButton.setDefault(True)
        self.runButton.setObjectName("runButton")
        self.horizontalLayout.addWidget(self.runButton)
        self.saveButton = QtWidgets.QPushButton(Dialog)
        self.saveButton.setObjectName("saveButton")
        self.horizontalLayout.addWidget(self.saveButton)
        self.cancelButton = QtWidgets.QPushButton(Dialog)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout.addWidget(self.cancelButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Settings"))
        self.label.setText(_translate("Dialog", "FreeSpace 2 Open build"))
        self.browseButton.setText(_translate("Dialog", "Browse"))
        self.label_2.setText(_translate("Dialog", "Video"))
        self.resolutionLabel.setText(_translate("Dialog", "Resolution :"))
        self.depthLabel.setText(_translate("Dialog", "Color depth :"))
        self.textureFilterLabel.setText(_translate("Dialog", "Texture filter :"))
        self.antialiasingLabel.setText(_translate("Dialog", "Antialiasing :"))
        self.anisotropicFilterLabel.setText(_translate("Dialog", "Anisotropic filtering :"))
        self.label_3.setText(_translate("Dialog", "Audio"))
        self.playbackDeviceLabel.setText(_translate("Dialog", "Playback device :"))
        self.captureDeviceLabel.setText(_translate("Dialog", "Capture device :"))
        self.snd_efx.setText(_translate("Dialog", "Enable EFX"))
        self.sampleRateLabel.setText(_translate("Dialog", "Sample rate :"))
        self.keyboardLabel.setText(_translate("Dialog", "Keyboard"))
        self.label_8.setText(_translate("Dialog", "Layout: "))
        self.setxkbmapCheck.setText(_translate("Dialog", "Use \"setxbmap\""))
        self.label_4.setText(_translate("Dialog", "Controller"))
        self.label_6.setText(_translate("Dialog", "Network"))
        self.connectionTypeLabel.setText(_translate("Dialog", "Connection type :"))
        self.connectionSpeedLabel.setText(_translate("Dialog", "Connection speed :"))
        self.forceLocalPortLabel.setText(_translate("Dialog", "Force local port :"))
        self.forceIPAddressLabel.setText(_translate("Dialog", "Force IP address :"))
        self.label_5.setText(_translate("Dialog", "Advanced settings"))
        self.cmdButton.setText(_translate("Dialog", "Command line flags for Mod name"))
        self.runButton.setText(_translate("Dialog", "Launch"))
        self.saveButton.setText(_translate("Dialog", "Save"))
        self.cancelButton.setText(_translate("Dialog", "Close"))

