# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/mod_settings.ui'
#
# Created by: PyQt5 UI code generator 5.7.1
#
# WARNING! All changes made in this file will be lost!

from ..qt import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(457, 552)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.modTitle = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.modTitle.setFont(font)
        self.modTitle.setAlignment(QtCore.Qt.AlignCenter)
        self.modTitle.setObjectName("modTitle")
        self.verticalLayout.addWidget(self.modTitle)
        self.tabWidget = QtWidgets.QTabWidget(Dialog)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.modLogo = QtWidgets.QLabel(self.tab)
        self.modLogo.setAlignment(QtCore.Qt.AlignCenter)
        self.modLogo.setObjectName("modLogo")
        self.verticalLayout_2.addWidget(self.modLogo)
        self.modDesc = QtWidgets.QPlainTextEdit(self.tab)
        self.modDesc.setReadOnly(True)
        self.modDesc.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
        self.modDesc.setObjectName("modDesc")
        self.verticalLayout_2.addWidget(self.modDesc)
        self.tabWidget.addTab(self.tab, "")
        self.flagsTab = QtWidgets.QWidget()
        self.flagsTab.setObjectName("flagsTab")
        self.tabWidget.addTab(self.flagsTab, "")
        self.pkgTab = QtWidgets.QWidget()
        self.pkgTab.setObjectName("pkgTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.pkgTab)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.pkgsLayout = QtWidgets.QVBoxLayout()
        self.pkgsLayout.setObjectName("pkgsLayout")
        self.verticalLayout_3.addLayout(self.pkgsLayout)
        self.dlSizeLabel = QtWidgets.QLabel(self.pkgTab)
        self.dlSizeLabel.setObjectName("dlSizeLabel")
        self.verticalLayout_3.addWidget(self.dlSizeLabel)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.applyPkgChanges = QtWidgets.QPushButton(self.pkgTab)
        self.applyPkgChanges.setObjectName("applyPkgChanges")
        self.verticalLayout_3.addWidget(self.applyPkgChanges)
        self.tabWidget.addTab(self.pkgTab, "")
        self.versionsTab = QtWidgets.QWidget()
        self.versionsTab.setObjectName("versionsTab")
        self.tabWidget.addTab(self.versionsTab, "")
        self.troubleTab = QtWidgets.QWidget()
        self.troubleTab.setObjectName("troubleTab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.troubleTab)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.checkFiles = QtWidgets.QPushButton(self.troubleTab)
        self.checkFiles.setObjectName("checkFiles")
        self.horizontalLayout.addWidget(self.checkFiles)
        self.delLoose = QtWidgets.QPushButton(self.troubleTab)
        self.delLoose.setObjectName("delLoose")
        self.horizontalLayout.addWidget(self.delLoose)
        self.replaceFiles = QtWidgets.QPushButton(self.troubleTab)
        self.replaceFiles.setObjectName("replaceFiles")
        self.horizontalLayout.addWidget(self.replaceFiles)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.logDisplay = QtWidgets.QTextBrowser(self.troubleTab)
        self.logDisplay.setObjectName("logDisplay")
        self.verticalLayout_4.addWidget(self.logDisplay)
        self.tabWidget.addTab(self.troubleTab, "")
        self.verticalLayout.addWidget(self.tabWidget)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(4)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Mod settings"))
        self.modTitle.setText(_translate("Dialog", "TextLabel"))
        self.modLogo.setText(_translate("Dialog", "TextLabel"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("Dialog", "Infos"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.flagsTab), _translate("Dialog", "Flags"))
        self.dlSizeLabel.setText(_translate("Dialog", "Download size: {DL_SIZE}"))
        self.applyPkgChanges.setText(_translate("Dialog", "Apply changes"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.pkgTab), _translate("Dialog", "Packages"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.versionsTab), _translate("Dialog", "Versions"))
        self.checkFiles.setText(_translate("Dialog", "Check Files"))
        self.delLoose.setText(_translate("Dialog", "Delete loose files"))
        self.replaceFiles.setText(_translate("Dialog", "Replace corrupted files"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.troubleTab), _translate("Dialog", "Troubleshoot"))

